from argparse import Namespace
from pathlib import Path
from typing import Iterator


def load_scope(args: Namespace, config: dict) -> Iterator[Path]:
    """Load a list of files in scope based on args and config"""
    src = Path(args.src)
    if src.is_file():
        yield src

    else:
        excluded = config.exclude.files
        for include in config.include.files:
            for path in src.rglob(include):
                if excluded and excluded.match(path.as_posix()):
                    continue

                yield path



import json
from argparse import Namespace

from whispers.models.pair import KeyValuePair


def printer(args: Namespace, pair: KeyValuePair) -> str:
    """Prints formatted pair data to given output"""
    fmt = json.dumps(
        {
            "key": pair.key,
            "value": pair.value,
            "file": pair.file,
            "line": pair.line,
            "rule_id": pair.rule.id,
            "message": pair.rule.message,
            "severity": pair.rule.severity,
        }
    )

    print(fmt, file=args.output)
    return fmt



from dataclasses import dataclass, field


@dataclass
class KeyValuePair:
    """Main key-value pair class"""

    key: str
    value: str
    keypath: list = field(default_factory=list)
    file: str = ""
    line: int = 0
    rule: dict = field(default_factory=dict)

    def __post_init__(self) -> None:
        if self.keypath == []:
            self.keypath = [self.key]

    def __repr__(self) -> str:
        return f"{self.key} = {self.value}"



from pathlib import Path
from typing import Iterator, Optional

from whispers.core.utils import is_uri, strip_string
from whispers.models.pair import KeyValuePair
from whispers.plugins.uri import Uri


class Plaintext:
    def pairs(self, filepath: Path) -> Iterator[KeyValuePair]:
        for lineno, line in enumerate(filepath.open(), 1):
            line = strip_string(line)
            if not line:
                continue

            yield from self.uri_pairs(line, lineno)
            yield from self.privatekey_pairs(line, lineno)

    @staticmethod
    def privatekey_pairs(line: str, lineno: int) -> Optional[Iterator[KeyValuePair]]:
        if not (line.startswith("---") and line.endswith("---")):
            return None

        if len(line) < 12:
            return None

        yield KeyValuePair("key", line, line=lineno)

    @staticmethod
    def uri_pairs(line: str, lineno: int) -> Optional[Iterator[KeyValuePair]]:
        if "://" not in line:
            return None

        for value in line.split():
            if not is_uri(value):
                continue

            for pair in Uri().pairs(value):
                pair.line = lineno
                yield pair



from typing import Iterator
from urllib.parse import parse_qsl, urlparse

from whispers.models.pair import KeyValuePair


class Uri:
    def pairs(self, code: str) -> Iterator[KeyValuePair]:
        uri = urlparse(code)
        if uri.password:
            yield KeyValuePair("uri creds", f"{uri.username}:{uri.password}", [code])

        if uri.query:
            for key, value in parse_qsl(uri.query):
                yield KeyValuePair(key, value, [code])
